==> osm2go_0.3-2.tar.gz: osm2go-0.3/canvas_goocanvas.c <==
/*
 * Copyright (C) 2008 Till Harbaum <till@harbaum.org>.
 *
 * This file is part of OSM2Go.
 *
 * OSM2Go is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OSM2Go is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSM2Go.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef USE_GOOCANVAS
#error "Config error!"
#endif

#include "appdata.h"

canvas_item_t *canvas_circle_new(map_t *map, canvas_group_t group, 
			 gint x, gint y, gint radius, gint border, 
			 canvas_color_t fill_col, canvas_color_t border_col) {
  return goo_canvas_ellipse_new(map->group[group],
				(gdouble) x, (gdouble) y,
				(gdouble) radius, (gdouble) radius,
				"line-width", (double)border,
				"stroke-color-rgba", border_col,
				"fill-color-rgba", fill_col,
				NULL);
}

canvas_points_t *canvas_points_new(gint points) {
  return goo_canvas_points_new(points);
}

void canvas_point_set_pos(canvas_points_t *points, gint index, lpos_t *lpos) {
  points->coords[2*index+0] = lpos->x;
  points->coords[2*index+1] = lpos->y;
}

void canvas_points_free(canvas_points_t *points) {
  goo_canvas_points_unref(points);
}

canvas_item_t *canvas_polyline_new(struct map_s *map, canvas_group_t group, 
		  canvas_points_t *points, gint width, canvas_color_t color) {
  return goo_canvas_polyline_new(map->group[group], FALSE, 0,
				 "points", points,
				 "line-width", (double)width,
				 "stroke-color-rgba", color,
				 "line-join", CAIRO_LINE_JOIN_ROUND,
				 "line-cap", CAIRO_LINE_CAP_ROUND,
				 NULL);
}

canvas_item_t *canvas_polygon_new(struct map_s *map, canvas_group_t group, 
		  canvas_points_t *points, gint width, canvas_color_t color,
				  canvas_color_t fill) {
  return goo_canvas_polyline_new(map->group[group], TRUE, 0,
				 "points", points,
				 "line-width", (double)width,
				 "stroke-color-rgba", color,
				 "fill-color-rgba", fill,
				 "line-join", CAIRO_LINE_JOIN_ROUND,
				 "line-cap", CAIRO_LINE_CAP_ROUND,
				 NULL);
}

void canvas_item_set_points(canvas_item_t *item, canvas_points_t *points) {
  g_object_set(G_OBJECT(item), "points", points, NULL);
}

void canvas_item_set_pos(canvas_item_t *item, lpos_t *lpos, gint radius) {
  g_object_set(G_OBJECT(item), "center-x", (gdouble)lpos->x, 
	       "center-y", (gdouble)lpos->y, NULL);  
}

void canvas_window2world(canvas_t *canvas, gint x, gint y, gint *wx, gint *wy) {
  double sx = x, sy = y;
  goo_canvas_convert_from_pixels(GOO_CANVAS(canvas), &sx, &sy);
  *wx = sx; *wy = sy;
}

canvas_item_t *canvas_get_item_at(canvas_t *canvas, gint x, gint y) {
  return goo_canvas_get_item_at(GOO_CANVAS(canvas), x, y, TRUE);
}

void canvas_item_to_bottom(canvas_item_t *item) {
  goo_canvas_item_lower(item, NULL);
}

void canvas_item_destroy(canvas_item_t *item) {
  goo_canvas_item_remove(item);
}

void canvas_item_set_user_data(canvas_item_t *item, void *data) {
  g_object_set_data(G_OBJECT(item), "user data", data);
}

void *canvas_item_get_user_data(canvas_item_t *item) {
  return g_object_get_data(G_OBJECT(item), "user data");
}

typedef struct {
  GCallback c_handler;
  gpointer data;
} weak_t;

static void canvas_item_weak_notify(gpointer data, GObject *invalid) {
  weak_t *weak = data;

  ((void(*)(GtkWidget*, gpointer))weak->c_handler) (NULL, weak->data);
  g_free(weak);
}

void canvas_item_destroy_connect(canvas_item_t *item, 
				 GCallback c_handler, gpointer data) {
  weak_t *weak = g_new(weak_t,1);
  weak->data = data;
  weak->c_handler = c_handler;

  g_object_weak_ref(G_OBJECT(item), canvas_item_weak_notify, weak);
}

void canvas_set_zoom(canvas_t *canvas, double zoom) {
  goo_canvas_set_scale(GOO_CANVAS(canvas), zoom);
}

void canvas_get_scroll_offsets(canvas_t *canvas, gint *sx, gint *sy) {
  GtkAdjustment *hadj = ((struct _GooCanvas*)canvas)->hadjustment;
  GtkAdjustment *vadj = ((struct _GooCanvas*)canvas)->vadjustment;
  gdouble hs, vs;
  gdouble zoom = goo_canvas_get_scale(GOO_CANVAS(canvas));

  hs = gtk_adjustment_get_value(hadj);
  vs = gtk_adjustment_get_value(vadj);
  goo_canvas_convert_from_pixels(GOO_CANVAS(canvas), &hs, &vs);

  /* make values zoom independant */
  *sx = hs * zoom;
  *sy = vs * zoom;
}

void canvas_scroll_to(canvas_t *canvas, gint sx, gint sy) {
  gdouble zoom = goo_canvas_get_scale(GOO_CANVAS(canvas));
  goo_canvas_scroll_to(GOO_CANVAS(canvas), sx/zoom, sy/zoom);
}

void canvas_set_bounds(canvas_t *canvas, gint minx, gint miny, 
		       gint maxx, gint maxy) {
  goo_canvas_set_bounds(GOO_CANVAS(canvas), minx, miny, maxx, maxy);
}

==> osm2go_0.3-3.tar.gz: osm2go-0.3/canvas_goocanvas.c <==
/*
 * Copyright (C) 2008 Till Harbaum <till@harbaum.org>.
 *
 * This file is part of OSM2Go.
 *
 * OSM2Go is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * OSM2Go is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OSM2Go.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef USE_GOOCANVAS
#error "Config error!"
#endif

#include "appdata.h"

canvas_item_t *canvas_circle_new(map_t *map, canvas_group_t group, 
			 gint x, gint y, gint radius, gint border, 
			 canvas_color_t fill_col, canvas_color_t border_col) {
  return goo_canvas_ellipse_new(map->group[group],
				(gdouble) x, (gdouble) y,
				(gdouble) radius, (gdouble) radius,
				"line-width", (double)border,
				"stroke-color-rgba", border_col,
				"fill-color-rgba", fill_col,
				NULL);
}

canvas_points_t *canvas_points_new(gint points) {
  return goo_canvas_points_new(points);
}

void canvas_point_set_pos(canvas_points_t *points, gint index, lpos_t *lpos) {
  points->coords[2*index+0] = lpos->x;
  points->coords[2*index+1] = lpos->y;
}

void canvas_points_free(canvas_points_t *points) {
  goo_canvas_points_unref(points);
}

canvas_item_t *canvas_polyline_new(struct map_s *map, canvas_group_t group, 
		  canvas_points_t *points, gint width, canvas_color_t color) {
  return goo_canvas_polyline_new(map->group[group], FALSE, 0,
				 "points", points,
				 "line-width", (double)width,
				 "stroke-color-rgba", color,
				 "line-join", CAIRO_LINE_JOIN_ROUND,
				 "line-cap", CAIRO_LINE_CAP_ROUND,
				 NULL);
}

canvas_item_t *canvas_polygon_new(struct map_s *map, canvas_group_t group, 
		  canvas_points_t *points, gint width, canvas_color_t color,
				  canvas_color_t fill) {
  return goo_canvas_polyline_new(map->group[group], TRUE, 0,
				 "points", points,
				 "line-width", (double)width,
				 "stroke-color-rgba", color,
				 "fill-color-rgba", fill,
				 "line-join", CAIRO_LINE_JOIN_ROUND,
				 "line-cap", CAIRO_LINE_CAP_ROUND,
				 NULL);
}

void canvas_item_set_points(canvas_item_t *item, canvas_points_t *points) {
  g_object_set(G_OBJECT(item), "points", points, NULL);
}

void canvas_item_set_pos(canvas_item_t *item, lpos_t *lpos, gint radius) {
  g_object_set(G_OBJECT(item), "center-x", (gdouble)lpos->x, 
	       "center-y", (gdouble)lpos->y, NULL);  
}

void canvas_window2world(canvas_t *canvas, gint x, gint y, gint *wx, gint *wy) {
  double sx = x, sy = y;
  goo_canvas_convert_from_pixels(GOO_CANVAS(canvas), &sx, &sy);
  *wx = sx; *wy = sy;
}

canvas_item_t *canvas_get_item_at(canvas_t *canvas, gint x, gint y) {
  return goo_canvas_get_item_at(GOO_CANVAS(canvas), x, y, TRUE);
}

void canvas_item_to_bottom(canvas_item_t *item) {
  goo_canvas_item_lower(item, NULL);
}

void canvas_item_destroy(canvas_item_t *item) {
  goo_canvas_item_remove(item);
}

void canvas_item_set_user_data(canvas_item_t *item, void *data) {
  g_object_set_data(G_OBJECT(item), "user data", data);
}

void *canvas_item_get_user_data(canvas_item_t *item) {
  return g_object_get_data(G_OBJECT(item), "user data");
}

typedef struct {
  GCallback c_handler;
  gpointer data;
} weak_t;

static void canvas_item_weak_notify(gpointer data, GObject *invalid) {
  weak_t *weak = data;

  ((void(*)(GtkWidget*, gpointer))weak->c_handler) (NULL, weak->data);
  g_free(weak);
}

void canvas_item_destroy_connect(canvas_item_t *item, 
				 GCallback c_handler, gpointer data) {
  weak_t *weak = g_new(weak_t,1);
  weak->data = data;
  weak->c_handler = c_handler;

  g_object_weak_ref(G_OBJECT(item), canvas_item_weak_notify, weak);
}

void canvas_set_zoom(canvas_t *canvas, double zoom) {
  goo_canvas_set_scale(GOO_CANVAS(canvas), zoom);
}

void canvas_get_scroll_offsets(canvas_t *canvas, gint *sx, gint *sy) {
  GtkAdjustment *hadj = ((struct _GooCanvas*)canvas)->hadjustment;
  GtkAdjustment *vadj = ((struct _GooCanvas*)canvas)->vadjustment;
  gdouble hs, vs;
  gdouble zoom = goo_canvas_get_scale(GOO_CANVAS(canvas));

  hs = gtk_adjustment_get_value(hadj);
  vs = gtk_adjustment_get_value(vadj);
  goo_canvas_convert_from_pixels(GOO_CANVAS(canvas), &hs, &vs);

  /* make values zoom independant */
  *sx = hs * zoom;
  *sy = vs * zoom;
}

void canvas_scroll_to(canvas_t *canvas, gint sx, gint sy) {
  gdouble zoom = goo_canvas_get_scale(GOO_CANVAS(canvas));
  goo_canvas_scroll_to(GOO_CANVAS(canvas), sx/zoom, sy/zoom);
}

void canvas_set_bounds(canvas_t *canvas, gint minx, gint miny, 
		       gint maxx, gint maxy) {
  goo_canvas_set_bounds(GOO_CANVAS(canvas), minx, miny, maxx, maxy);
}

canvas_item_t *canvas_image_new(map_t *map, canvas_group_t group, 
		GdkPixbuf *pix, gint x, gint y, float hscale, float vscale) {

  canvas_item_t *item = goo_canvas_image_new(map->group[group], pix, 
					     x/hscale, y/vscale, NULL);
  goo_canvas_item_scale(item, hscale, vscale);
  return item;
}
